"""

.. moduleauthor:: easygui developers and Stephen Raymond Ferg
.. default-domain:: py
.. highlight:: python

Version |release|
"""

try:
    from . import global_state
except:
    import global_state

try:
    import tkinter as tk  # python 3
except:
    import Tkinter as tk  # python 2

# -----------------------------------------------------------------------
# multpasswordbox
# -----------------------------------------------------------------------


def multpasswordbox(msg="Fill in values for the fields.",
                    title=" ", fields=tuple(), values=tuple(),
                    callback=None, run=True):
    r"""
    Same interface as multenterbox.  But in multpassword box,
    the last of the fields is assumed to be a password, and
    is masked with asterisks.

    :param str msg: the msg to be displayed.
    :param str title: the window title
    :param list fields: a list of fieldnames.
    :param list values: a list of field values
    :return: String

    **Example**

    Here is some example code, that shows how values returned from
    multpasswordbox can be checked for validity before they are accepted::

        msg = "Enter logon information"
        title = "Demo of multpasswordbox"
        fieldNames = ["Server ID", "User ID", "Password"]
        fieldValues = []  # we start with blanks for the values
        fieldValues = multpasswordbox(msg,title, fieldNames)

        # make sure that none of the fields was left blank
        while 1:
            if fieldValues is None: break
            errmsg = ""
            for i in range(len(fieldNames)):
                if fieldValues[i].strip() == "":
                    errmsg = errmsg + ('"%s" is a required field.\n\n' %
                     fieldNames[i])
                if errmsg == "": break # no problems found
            fieldValues = multpasswordbox(errmsg, title,
              fieldNames, fieldValues)

        print("Reply was: %s" % str(fieldValues))

    """
    if run:
        mb = MultiBox(msg, title, fields, values, mask_last=True,
                      callback=callback)

        reply = mb.run()

        return reply

    else:

        mb = MultiBox(msg, title, fields, values, mask_last=True,
                      callback=callback)

        return mb


# -------------------------------------------------------------------
# multenterbox
# -------------------------------------------------------------------
# TODO RL: Should defaults be list constructors.
# i think after multiple calls, the value is retained.
# TODO RL: Rename/alias to multienterbox?
# default should be None and then in the logic create an empty liglobal_state.
def multenterbox(msg="Fill in values for the fields.", title=" ",
                 fields=[], values=[], callback=None, run=True):
    r"""
    Show screen with multiple data entry fields.

    If there are fewer values than names, the list of values is padded with
    empty strings until the number of values is the same as the number
    of names.

    If there are more values than names, the list of values
    is truncated so that there are as many values as names.

    Returns a list of the values of the fields,
    or None if the user cancels the operation.

    Here is some example code, that shows how values returned from
    multenterbox can be checked for validity before they are accepted::

        msg = "Enter your personal information"
        title = "Credit Card Application"
        fieldNames = ["Name","Street Address","City","State","ZipCode"]
        fieldValues = []  # we start with blanks for the values
        fieldValues = multenterbox(msg,title, fieldNames)

        # make sure that none of the fields was left blank
        while 1:
            if fieldValues is None: break
            errmsg = ""
            for i in range(len(fieldNames)):
                if fieldValues[i].strip() == "":
                    errmsg += ('"%s" is a required field.\n\n' % fieldNames[i])
            if errmsg == "":
                break # no problems found
            fieldValues = multenterbox(errmsg, title, fieldNames, fieldValues)

        print("Reply was: %s" % str(fieldValues))

    :param str msg: the msg to be displayed.
    :param str title: the window title
    :param list fields: a list of fieldnames.
    :param list values: a list of field values
    :return: String
    """
    if run:
        mb = MultiBox(msg, title, fields, values, mask_last=False,
                      callback=callback)
        reply = mb.run()
        return reply
    else:
        mb = MultiBox(msg, title, fields, values, mask_last=False,
                      callback=callback)
        return mb


class MultiBox(object):

    """ Show multiple data entry fields

    This object does a number of things:

    - chooses a GUI framework (wx, qt)
    - checks the data sent to the GUI
    - performs the logic (button ok should close the window?)
    - defines what methods the user can invoke and
      what properties he can change.
    - calls the ui in defined ways, so other gui
      frameworks can be used without breaking anything to the user
    """

    def __init__(self, msg, title, fields, values, mask_last, callback):
        """ Create box object

        Parameters
        ----------
        msg : string
            text displayed in the message area (instructions...)
        title : str
            the window title
        fields: list
            names of fields
        values: list
            initial values
        callback: function
            if set, this function will be called when OK is pressed
        run: bool
            if True, a box object will be created and returned, but not run

        Returns
        -------
        self
            The MultiBox object
        """

        self.callback = callback

        self.fields, self.values = self.check_fields(fields, values)

        self.ui = GUItk(msg, title, self.fields, self.values,
                        mask_last, self.callback_ui)

    def run(self):
        """ Start the ui """
        self.ui.run()
        self.ui = None
        return self.values

    def stop(self):
        """ Stop the ui """
        self.ui.stop()

    def callback_ui(self, ui, command, values):
        """ This method is executed when ok, cancel, or x is pressed in the ui.
        """
        if command == 'update':  # OK was pressed
            self.values = values
            if self.callback:
                # If a callback was set, call main process
                self.callback(self)
            else:
                self.stop()
        elif command == 'x':
            self.stop()
            self.values = None
        elif command == 'cancel':
            self.stop()
            self.values = None

    # methods to change properties --------------

    @property
    def msg(self):
        """Text in msg Area"""
        return self._msg

    @msg.setter
    def msg(self, msg):
        self.ui.set_msg(msg)

    @msg.deleter
    def msg(self):
        self._msg = ""
        self.ui.set_msg(self._msg)

    # Methods to validate what will be sent to ui ---------

    def check_fields(self, fields, values):
        if len(fields) == 0:
            return None

        fields = list(fields[:])  # convert possible tuples to a list
        values = list(values[:])  # convert possible tuples to a list

        # TODO RL: The following seems incorrect when values>fields.  Replace
        # below with zip?
        if len(values) == len(fields):
            pass
        elif len(values) > len(fields):
            fields = fields[0:len(values)]
        else:
            while len(values) < len(fields):
                values.append("")

        return fields, values


class GUItk(object):

    """ This object contains the tk root object.
        It draws the window, waits for events and communicates them
        to MultiBox, together with the entered values.

        The position in wich it is drawn comes from a global variable.

        It also accepts commands from Multibox to change its message.
    """

    def __init__(self, msg, title, fields, values, mask_last, callback):

        self.callback = callback

        self.boxRoot = tk.Tk()

        self.create_root(title)

        self.set_pos(global_state.window_position)  # GLOBAL POSITION

        self.create_msg_widget(msg)

        self.create_entryWidgets(fields, values, mask_last)

        self.create_buttons()

        self.entryWidgets[0].focus_force()  # put the focus on the entryWidget

    # Run and stop methods ---------------------------------------

    def run(self):
        self.boxRoot.mainloop()  # run it!
        self.boxRoot.destroy()   # Close the window

    def stop(self):
        # Get the current position before quitting
        self.get_pos()

        self.boxRoot.quit()

    def x_pressed(self):
        self.callback(self, command='x', values=self.get_values())

    def cancel_pressed(self, event):
        self.callback(self, command='cancel', values=self.get_values())

    def ok_pressed(self, event):
        self.callback(self, command='update', values=self.get_values())

    # Methods to change content ---------------------------------------

    def set_msg(self, msg):
        self.messageWidget.configure(text=msg)
        self.entryWidgets[0].focus_force()  # put the focus on the entryWidget

    def set_pos(self, pos):
        self.boxRoot.geometry(pos)

    def get_pos(self):
        # The geometry() method sets a size for the window and positions it on
        # the screen. The first two parameters are width and height of
        # the window. The last two parameters are x and y screen coordinates.
        # geometry("250x150+300+300")
        geom = self.boxRoot.geometry()  # "628x672+300+200"
        global_state.window_position = '+' + geom.split('+', 1)[1]

    def get_values(self):
        values = []
        for entryWidget in self.entryWidgets:
            values.append(entryWidget.get())
        return values

    # Initial configuration methods ---------------------------------------
    # These ones are just called once, at setting.

    def create_root(self, title):

        self.boxRoot.protocol('WM_DELETE_WINDOW', self.x_pressed)
        self.boxRoot.title(title)
        self.boxRoot.iconname('Dialog')
        self.boxRoot.bind("<Escape>", self.cancel_pressed)

    def create_msg_widget(self, msg):
        # -------------------- the msg widget ----------------------------
        self.messageWidget = tk.Message(self.boxRoot, width="4.5i", text=msg)
        self.messageWidget.configure(
            font=(global_state.PROPORTIONAL_FONT_FAMILY, global_state.PROPORTIONAL_FONT_SIZE))
        self.messageWidget.pack(
            side=tk.TOP, expand=1, fill=tk.BOTH, padx='3m', pady='3m')

    def create_entryWidgets(self, fields, values, mask_last):

        self.entryWidgets = []

        lastWidgetIndex = len(fields) - 1

        for widgetIndex in range(len(fields)):
            name = fields[widgetIndex]
            value = values[widgetIndex]
            entryFrame = tk.Frame(master=self.boxRoot)
            entryFrame.pack(side=tk.TOP, fill=tk.BOTH)

            # --------- entryWidget -------------------------------------------
            labelWidget = tk.Label(entryFrame, text=name)
            labelWidget.pack(side=tk.LEFT)

            entryWidget = tk.Entry(entryFrame, width=40, highlightthickness=2)
            self.entryWidgets.append(entryWidget)
            entryWidget.configure(
                font=(global_state.PROPORTIONAL_FONT_FAMILY, global_state.TEXT_ENTRY_FONT_SIZE))
            entryWidget.pack(side=tk.RIGHT, padx="3m")

            self.bindArrows(entryWidget)

            entryWidget.bind("<Return>", self.ok_pressed)
            entryWidget.bind("<Escape>", self.cancel_pressed)

            # for the last entryWidget, if this is a multpasswordbox,
            # show the contents as just asterisks
            if widgetIndex == lastWidgetIndex:
                if mask_last:
                    self.entryWidgets[widgetIndex].configure(show="*")

            # put text into the entryWidget
            if value is None:
                value = ''
            self.entryWidgets[widgetIndex].insert(
                0, '{}'.format(value))

    def create_buttons(self):
        self.buttonsFrame = tk.Frame(master=self.boxRoot)
        self.buttonsFrame.pack(side=tk.BOTTOM)

        self.create_cancel_button()
        self.create_ok_button()

    def create_ok_button(self):

        okButton = tk.Button(self.buttonsFrame, takefocus=1, text="OK")
        self.bindArrows(okButton)
        okButton.pack(expand=1, side=tk.LEFT, padx='3m', pady='3m',
                      ipadx='2m', ipady='1m')

        # for the commandButton, bind activation events to the activation event
        # handler
        commandButton = okButton
        handler = self.ok_pressed
        for selectionEvent in global_state.STANDARD_SELECTION_EVENTS:
            commandButton.bind("<%s>" % selectionEvent, handler)

    def create_cancel_button(self):

        cancelButton = tk.Button(self.buttonsFrame, takefocus=1, text="Cancel")
        self.bindArrows(cancelButton)
        cancelButton.pack(expand=1, side=tk.LEFT, padx='3m', pady='3m',
                          ipadx='2m', ipady='1m')

        # for the commandButton, bind activation events to the activation event
        # handler
        commandButton = cancelButton
        handler = self.cancel_pressed
        for selectionEvent in global_state.STANDARD_SELECTION_EVENTS:
            commandButton.bind("<%s>" % selectionEvent, handler)

    def bindArrows(self, widget):

        widget.bind("<Down>", self.tabRight)
        widget.bind("<Up>", self.tabLeft)

        widget.bind("<Right>", self.tabRight)
        widget.bind("<Left>", self.tabLeft)

    def tabRight(self, event):
        self.boxRoot.event_generate("<Tab>")

    def tabLeft(self, event):
        self.boxRoot.event_generate("<Shift-Tab>")


def demo1():
    msg = "Enter your personal information"
    title = "Credit Card Application"
    fieldNames = ["Name", "Street Address", "City", "State", "ZipCode"]
    fieldValues = []  # we start with blanks for the values

    # make sure that none of the fields was left blank
    while True:

        fieldValues = multenterbox(msg, title, fieldNames, fieldValues)
        cancelled = fieldValues is None
        errors = []
        if cancelled:
            pass
        else:  # check for errors
            for name, value in zip(fieldNames, fieldValues):
                if value.strip() == "":
                    errors.append('"{}" is a required field.'.format(name))

        all_ok = not errors

        if cancelled or all_ok:
            break  # no problems found

        msg = "\n".join(errors)

    print("Reply was: {}".format(fieldValues))


class Demo2():

    def __init__(self):
        msg = "Without flicker. Enter your personal information"
        title = "Credit Card Application"
        fieldNames = ["Name", "Street Address", "City", "State", "ZipCode"]
        fieldValues = []  # we start with blanks for the values

        fieldValues = multenterbox(msg, title, fieldNames, fieldValues,
                                   callback=self.check_for_blank_fields)
        print("Reply was: {}".format(fieldValues))

    def check_for_blank_fields(self, box):
        # make sure that none of the fields was left blank
        cancelled = box.values is None
        errors = []
        if cancelled:
            pass
        else:  # check for errors
            for name, value in zip(box.fields, box.values):
                if value.strip() == "":
                    errors.append('"{}" is a required field.'.format(name))

        all_ok = not errors

        if cancelled or all_ok:
            box.stop()  # no problems found

        box.msg = "\n".join(errors)


if __name__ == '__main__':
    demo1()
    Demo2()



"""

.. moduleauthor:: easygui developers and Stephen Raymond Ferg
.. default-domain:: py
.. highlight:: python

Version |release|
"""

boxRoot = None


def bindArrows(widget):

    widget.bind("<Down>", tabRight)
    widget.bind("<Up>", tabLeft)

    widget.bind("<Right>", tabRight)
    widget.bind("<Left>", tabLeft)


def tabRight(event):
    boxRoot.event_generate("<Tab>")


def tabLeft(event):
    boxRoot.event_generate("<Shift-Tab>")



from __future__ import print_function
"""

.. moduleauthor:: easygui developers and Stephen Raymond Ferg
.. default-domain:: py
.. highlight:: python

Version |release|
"""

import os
try:
    from . import utils as ut
    from . import fileboxsetup as fbs
except (SystemError, ValueError, ImportError):
    import utils as ut
    import fileboxsetup as fbs

tk = ut.tk



# -------------------------------------------------------------------
# fileopenbox
# -------------------------------------------------------------------


def fileopenbox(msg=None, title=None, default='*', filetypes=None, multiple=False):
    """
    A dialog to get a file name.

    **About the "default" argument**

    The "default" argument specifies a filepath that (normally)
    contains one or more wildcards.
    fileopenbox will display only files that match the default filepath.
    If omitted, defaults to "\*" (all files in the current directory).

    WINDOWS EXAMPLE::

        ...default="c:/myjunk/*.py"

    will open in directory c:\\myjunk\\ and show all Python files.

    WINDOWS EXAMPLE::

        ...default="c:/myjunk/test*.py"

    will open in directory c:\\myjunk\\ and show all Python files
    whose names begin with "test".


    Note that on Windows, fileopenbox automatically changes the path
    separator to the Windows path separator (backslash).

    **About the "filetypes" argument**

    If specified, it should contain a list of items,
    where each item is either:

    - a string containing a filemask          # e.g. "\*.txt"
    - a list of strings, where all of the strings except the last one
      are filemasks (each beginning with "\*.",
      such as "\*.txt" for text files, "\*.py" for Python files, etc.).
      and the last string contains a filetype description

    EXAMPLE::

        filetypes = ["*.css", ["*.htm", "*.html", "HTML files"]  ]

    .. note:: If the filetypes list does not contain ("All files","*"), it will be added.

    If the filetypes list does not contain a filemask that includes
    the extension of the "default" argument, it will be added.
    For example, if default="\*abc.py"
    and no filetypes argument was specified, then
    "\*.py" will automatically be added to the filetypes argument.

    :param str msg: the msg to be displayed.
    :param str title: the window title
    :param str default: filepath with wildcards
    :param object filetypes: filemasks that a user can choose, e.g. "\*.txt"
    :param bool multiple: If true, more than one file can be selected
    :return: the name of a file, or None if user chose to cancel
    """
    localRoot = tk.Tk()
    localRoot.withdraw()

    initialbase, initialfile, initialdir, filetypes = fbs.fileboxSetup(
        default, filetypes)

    # ------------------------------------------------------------
    # if initialfile contains no wildcards; we don't want an
    # initial file. It won't be used anyway.
    # Also: if initialbase is simply "*", we don't want an
    # initialfile; it is not doing any useful work.
    # ------------------------------------------------------------
    if (initialfile.find("*") < 0) and (initialfile.find("?") < 0):
        initialfile = None
    elif initialbase == "*":
        initialfile = None

    func = ut.tk_FileDialog.askopenfilenames if multiple else ut.tk_FileDialog.askopenfilename
    ret_val = func(parent=localRoot,
                   title=ut.getFileDialogTitle(msg, title),
                   initialdir=initialdir, initialfile=initialfile,
                   filetypes=filetypes
                   )
    if not ret_val or ret_val == '':
        return None
    if multiple:
        f = [os.path.normpath(x) for x in localRoot.tk.splitlist(ret_val)]
    else:
        try:
            f = os.path.normpath(ret_val)
        except AttributeError as e:
            print("ret_val is {}".format(ret_val))
            raise e
    localRoot.destroy()

    if not f:
        return None
    return f


if __name__ == '__main__':
    print("Hello from file open box")
    ret_val = fileopenbox("Please select a file", "My File Open dialog")
    print("Return value is:{}".format(ret_val))



"""

.. moduleauthor:: easygui developers and Stephen Raymond Ferg
.. default-domain:: py
.. highlight:: python

Version |release|
"""

# Starting and global variables

window_position = "+300+200"

PROPORTIONAL_FONT_FAMILY = ('Arial',"MS", "Sans", "Serif")
MONOSPACE_FONT_FAMILY = "Arial"

# PROPORTIONAL_FONT_SIZE = 10
PROPORTIONAL_FONT_SIZE = 30
# a little smaller, because it is more legible at a smaller size
# MONOSPACE_FONT_SIZE = 9
MONOSPACE_FONT_SIZE = 50
# TEXT_ENTRY_FONT_SIZE = 12  # a little larger makes it easier to see
TEXT_ENTRY_FONT_SIZE = 30  # a little larger makes it easier to see


STANDARD_SELECTION_EVENTS = ["Return", "Button-1", "space"]

# prop_font_line_length = 62
# fixw_font_line_length = 80
# num_lines_displayed = 50
# default_hpad_in_chars = 2


prop_font_line_length = 50
fixw_font_line_length = 80
num_lines_displayed = 50
default_hpad_in_chars = 3



"""

.. moduleauthor:: easygui developers and Stephen Raymond Ferg
.. default-domain:: py
.. highlight:: python

Version |release|

"""

import os
import sys
import traceback

# A set of variables and functions to centralize differences between
# python 2 and 3
runningPython27 = False
runningPython34 = False
if 0x020700F0 <= sys.hexversion <= 0x030000F0:
    runningPython27 = True
if 0x030400F0 <= sys.hexversion <= 0x040000F0:
    runningPython34 = True
if not runningPython27 and not runningPython34:
    raise Exception("You must run on Python 2.7+ or Python 3.4+")

# Import Tkinter, the tk filedialog, and put everything in tkinter into
# the current namespace
try:
    import tkinter as tk  # python3
    # TODO: Ultimately this should go away once everything stops using it.
    from tkinter import *
    import tkinter.filedialog as tk_FileDialog
    import tkinter.font as tk_Font
except ImportError:
    try:
        import Tkinter as tk  # python2
        # TODO: Ultimately this should go away once everything stops using it.
        from Tkinter import *
        import tkFileDialog as tk_FileDialog
        import tkFont as tk_Font

    except ImportError:
        raise ImportError("Unable to find tkinter package.")

if tk.TkVersion < 8.0:
    raise ImportError("You must use python-tk (tkinter) version 8.0 or higher")


# Try to import the Python Image Library.  If it doesn't exist, only .gif
# images are supported.
try:
    from PIL import Image as PILImage
    from PIL import ImageTk as PILImageTk
except:
    pass

# Code should use 'basestring' anywhere you might think to use the system 'str'.  This is all to support
# Python 2.  If 2 ever goes away, this logic can go away and uses of
# utils.basestring should be changed to just str
if runningPython27:
    basestring = basestring
if runningPython34:
    basestring = str


def lower_case_sort(things):
    if runningPython34:
        things.sort(key=str.lower)
    else:
        # case-insensitive sort
        things.sort(lambda x, y: cmp(x.lower(), y.lower()))
    return things  # RL: Not sure of this exactly


# -----------------------------------------------------------------------
# exception_format
# -----------------------------------------------------------------------
def exception_format():
    """
    Convert exception info into a string suitable for display.
    """
    return "".join(traceback.format_exception(
        sys.exc_info()[0], sys.exc_info()[1], sys.exc_info()[2]
    ))


# -------------------------------------------------------------------
# utility routines
# -------------------------------------------------------------------
# These routines are used by several other functions in the EasyGui module.

def uniquify_list_of_strings(input_list):
    """
    Ensure that every string within input_list is unique.
    :param list input_list: List of strings
    :return: New list with unique names as needed.
    """
    output_list = list()
    for i, item in enumerate(input_list):
        tempList = input_list[:i] + input_list[i + 1:]
        if item not in tempList:
            output_list.append(item)
        else:
            output_list.append('{0}_{1}'.format(item, i))
    return output_list

import re


def parse_hotkey(text):
    """
    Extract a desired hotkey from the text.  The format to enclose
    the hotkey in square braces
    as in Button_[1] which would assign the keyboard key 1 to that button.
      The one will be included in the
    button text.  To hide they key, use double square braces as in:  Ex[[qq]]
    it  , which would assign
    the q key to the Exit button. Special keys such as <Enter> may also be
    used:  Move [<left>]  for a full
    list of special keys, see this reference: http://infohoglobal_state.nmt.edu/tcc/help/
    pubs/tkinter/web/key-names.html
    :param text:
    :return: list containing cleaned text, hotkey, and hotkey position within
    cleaned text.
    """

    ret_val = [text, None, None]  # Default return values
    if text is None:
        return ret_val

    # Single character, remain visible
    res = re.search('(?<=\[).(?=\])', text)
    if res:
        start = res.start(0)
        end = res.end(0)
        caption = text[:start - 1] + text[start:end] + text[end + 1:]
        ret_val = [caption, text[start:end], start - 1]

    # Single character, hide it
    res = re.search('(?<=\[\[).(?=\]\])', text)
    if res:
        start = res.start(0)
        end = res.end(0)
        caption = text[:start - 2] + text[end + 2:]
        ret_val = [caption, text[start:end], None]

    # a Keysym.  Always hide it
    res = re.search('(?<=\[\<).+(?=\>\])', text)
    if res:
        start = res.start(0)
        end = res.end(0)
        caption = text[:start - 2] + text[end + 2:]
        ret_val = [caption, '<{}>'.format(text[start:end]), None]

    return ret_val


def load_tk_image(filename, tk_master=None):
    """
    Load in an image file and return as a tk Image.

    Loads an image.  If the PIL library is available use it.  otherwise use the tk method.

    NOTE: tk_master is required if there are more than one Tk() instances, which there are very often.
      REF: http://stackoverflow.com/a/23229091/2184122

    :param filename: image filename to load
    :param tk_master: root object (Tk())
    :return: tk Image object
    """

    if filename is None:
        return None

    if not os.path.isfile(filename):
        raise ValueError(
            'Image file {} does not exist.'.format(filename))

    tk_image = None

    filename = os.path.normpath(filename)
    _, ext = os.path.splitext(filename)

    try:
        pil_image = PILImage.open(filename)
        tk_image = PILImageTk.PhotoImage(pil_image, master=tk_master)
    except:
        try:
            # Fallback if PIL isn't available
            tk_image = tk.PhotoImage(file=filename, master=tk_master)
        except:
            msg = "Cannot load {}.  Check to make sure it is an image file.".format(
                filename)
            try:
                _ = PILImage
            except:
                msg += "\nPIL library isn't installed.  If it isn't installed, only .gif files can be used."
            raise ValueError(msg)
    return tk_image


# -------------------------------------------------------------------
# getFileDialogTitle
# -------------------------------------------------------------------
def getFileDialogTitle(msg, title):
    """
    Create nicely-formatted string based on arguments msg and title
    :param msg: the msg to be displayed
    :param title: the window title
    :return: None
    """
    if msg and title:
        return "%s - %s" % (title, msg)
    if msg and not title:
        return str(msg)
    if title and not msg:
        return str(title)
    return None  # no message and no title


if __name__ == '__main__':
    print("Hello from utils")



"""

.. moduleauthor:: easygui developers and Stephen Raymond Ferg
.. default-domain:: py
.. highlight:: python

"""
try:
    from .derived_boxes import codebox
except (SystemError, ValueError, ImportError):
    from derived_boxes import codebox

eg_version = '0.98.0-RELEASED'
egversion = eg_version


def abouteasygui():
    """
    shows the easygui revision history
    """
    codebox("About EasyGui\n{}".format(eg_version),
            "EasyGui", EASYGUI_ABOUT_INFORMATION)
    return None


EASYGUI_ABOUT_INFORMATION = '''
========================================================================
0.98.0
========================================================================
This is an exciting time for easygui.  We continue to make good progress with refactoring as
well as some enhancements and bug fixes here and there.

We would like to welcome Juanjo Corrales to the team.  He is responsible for lots of good new work
this release.  Of course we appreciate the work of everyone who contributed.

NOTE: I decided in this release to change the API a bit.  Please consult the function documentation for details.

BUG FIXES
---------
 * Made changes guessing at fixes to any IDLE problems.  Please report any problems found.

ENHANCEMENTS
------------
 * Refactored the easygui.py file into several smaller files to improve our ability to manage the code
 * Added callbacks to allow for more dynamic dialogs.  See the docs for usage.
 * Added class access to dialogs so properties may be changed.
 * Improved button boxes ability to resize during window resize by converting to Tkinter grid from packer.

KNOWN ISSUES
------------
 * (old) In the documentation, there were previous references to issues when using the IDLE IDE.  I haven't
   experienced those, but also didn't do anything to fix them, so they may still be there.  Please report
   any problems and we'll try to address them

OTHER CHANGES
-------------
 * Centralized the Python 2 versus Python 3 "compatibility layer" into boxes/utils.py

========================================================================
0.97.4
========================================================================
This is a minor bug-fix release to address python 3 import errors.

========================================================================
0.97.3
========================================================================
Some more fixes especially to fix Debian distro problems.

========================================================================
0.97.1 & 0.97.2 (2014-12-24)
========================================================================
Boring, embarrassing uploads to fix pypi install problems.

========================================================================
0.97(2014-12-20)
========================================================================
We are happy to release version 0.97 of easygui.  The intent of this release is to address some basic
functionality issues as well as improve easygui in the ways people have asked.

Robert Lugg (me) was searching for a GUI library for my python work.  I saw easygui and liked very much its
paradigm.  Stephen Ferg, the creator and developer of easygui, graciously allowed me to start development
back up.  With the help of Alexander Zawadzki, Horst Jens, and others I set a goal to release before the
end of 2014.

We rely on user feedback so please bring up problems, ideas, or just say how you are using easygui.

BUG FIXES
---------
 * sourceforge #4: easygui docs contain bad references to easygui_pydoc.html
 * sourceforge #6: no index.html in docs download file.  Updated to sphinx which as autolinking.
 * sourceforge #8: unicode issues with file*box.  Fixed all that I knew how.
 * sourceforge #12: Cannot Exit with 'X'.  Now X and escape either return "cancel_button", if set, or None

ENHANCEMENTS
------------
 * Added ability to specify default_choice and cancel_choice for button widgets (See API docs)
 * True and False are returned instead of 1 and 0 for several boxes
 * Allow user to map keyboard keys to buttons by enclosing a hotkey in square braces like: "Pick [M]e", which would assign
   keyboard key M to that button.  Double braces hide that character, and keysyms are allowed:
     [[q]]Exit    Would show Exit on the button, and the button would be controlled by the q key
     [<F1>]Help   Would show Help on the button, and the button would be controlled by the F1 function key
   NOTE: We are still working on the exact syntax of these key mappings as Enter, space, and arrows are already being
         used.
 * Escape and the windows 'X' button always work in buttonboxes.  Those return None in that case.
 * sourceforge #9: let fileopenbox open multiple files.  Added optional argument 'multiple'
 * Location of dialogs on screen is preserved.  This isn't perfect yet, but now, at least, the dialogs don't
   always reset to their default position!
 * added some, but not all of the bugs/enhancements developed by Robbie Brook:
   http://all-you-need-is-tech.blogspot.com/2013/01/improving-easygui-for-python.html

KNOWN ISSUES
------------
 * In the documentation, there were previous references to issues when using the IDLE IDE.  I haven't
   experienced those, but also didn't do anything to fix them, so they may still be there.  Please report
   any problems and we'll try to address them
 * I am fairly new to contributing to open source, so I don't understand packaging, pypi, etc.  There
   are likely problems as well as better ways to do things.  Again, I appreciate any help or guidance.

Other Changes (that you likely don't care about)
------------------------------------------------
 * Restructured loading of image files to try PIL first throw error if file doesn't exiglobal_state.
 * Converted docs to sphinx with just a bit of docteglobal_state.  Most content was retained from the old site, so
   there might be some redundancies still.  Please make any suggested improvements.
 * Set up a GitHub repository for development: https://github.com/robertlugg/easygui

EasyGui is licensed under what is generally known as
the "modified BSD license" (aka "revised BSD", "new BSD", "3-clause BSD").
This license is GPL-compatible but less restrictive than GPL.

========================================================================
0.96(2010-08-29)
========================================================================
This version fixes some problems with version independence.

BUG FIXES
------------------------------------------------------
 * A statement with Python 2.x-style exception-handling syntax raised
   a syntax error when running under Python 3.x.
   Thanks to David Williams for reporting this problem.

 * Under some circumstances, PIL was unable to display non-gif images
   that it should have been able to display.
   The cause appears to be non-version-independent import syntax.
   PIL modules are now imported with a version-independent syntax.
   Thanks to Horst Jens for reporting this problem.

LICENSE CHANGE
------------------------------------------------------
Starting with this version, EasyGui is licensed under what is generally known as
the "modified BSD license" (aka "revised BSD", "new BSD", "3-clause BSD").
This license is GPL-compatible but less restrictive than GPL.
Earlier versions were licensed under the Creative Commons Attribution License 2.0.


========================================================================
0.95(2010-06-12)
========================================================================

ENHANCEMENTS
------------------------------------------------------
 * Previous versions of EasyGui could display only .gif image files using the
   msgbox "image" argument. This version can now display all image-file formats
   supported by PIL the Python Imaging Library) if PIL is installed.
   If msgbox is asked to open a non-gif image file, it attempts to import
   PIL and to use PIL to convert the image file to a displayable format.
   If PIL cannot be imported (probably because PIL is not installed)
   EasyGui displays an error message saying that PIL must be installed in order
   to display the image file.

   Note that
   http://www.pythonware.com/products/pil/
   says that PIL doesn't yet support Python 3.x.


========================================================================
0.94(2010-06-06)
========================================================================

ENHANCEMENTS
------------------------------------------------------
 * The codebox and textbox functions now return the contents of the box, rather
   than simply the name of the button ("Yes").  This makes it possible to use
   codebox and textbox as data-entry widgets.  A big "thank you!" to Dominic
   Comtois for requesting this feature, patiently explaining his requirement,
   and helping to discover the tkinter techniques to implement it.

   NOTE THAT in theory this change breaks backward compatibility.  But because
   (in previous versions of EasyGui) the value returned by codebox and textbox
   was meaningless, no application should have been checking it.  So in actual
   practice, this change should not break backward compatibility.

 * Added support for SPACEBAR to command buttons.  Now, when keyboard
   focus is on a command button, a press of the SPACEBAR will act like
   a press of the ENTER key; it will activate the command button.

 * Added support for keyboard navigation with the arrow keys (up,down,left,right)
   to the fields and buttons in enterbox, multenterbox and multpasswordbox,
   and to the buttons in choicebox and all buttonboxes.

 * added highlightthickness=2 to entry fields in multenterbox and
   multpasswordbox.  Now it is easier to tell which entry field has
   keyboard focus.


BUG FIXES
------------------------------------------------------
 * In EgStore, the pickle file is now opened with "rb" and "wb" rather than
   with "r" and "w".  This change is necessary for compatibility with Python 3+.
   Thanks to Marshall Mattingly for reporting this problem and providing the fix.

 * In integerbox, the actual argument names did not match the names described
   in the docstring. Thanks to Daniel Zingaro of at University of Toronto for
   reporting this problem.

 * In integerbox, the "argLowerBound" and "argUpperBound" arguments have been
   renamed to "lowerbound" and "upperbound" and the docstring has been corrected.

   NOTE THAT THIS CHANGE TO THE ARGUMENT-NAMES BREAKS BACKWARD COMPATIBILITY.
   If argLowerBound or argUpperBound are used, an AssertionError with an
   explanatory error message is raised.

 * In choicebox, the signature to choicebox incorrectly showed choicebox as
   accepting a "buttons" argument.  The signature has been fixed.


========================================================================
0.93(2009-07-07)
========================================================================

ENHANCEMENTS
------------------------------------------------------

 * Added exceptionbox to display stack trace of exceptions

 * modified names of some font-related constants to make it
   easier to customize them


========================================================================
0.92(2009-06-22)
========================================================================

ENHANCEMENTS
------------------------------------------------------

 * Added EgStore class to to provide basic easy-to-use persistence.

BUG FIXES
------------------------------------------------------

 * Fixed a bug that was preventing Linux users from copying text out of
   a textbox and a codebox.  This was not a problem for Windows users.

'''

if __name__ == '__main__':
    abouteasygui()


import collections
import string

try:
    from . import global_state
    from .base_boxes import bindArrows
except (SystemError, ValueError, ImportError):
    import global_state
    from base_boxes import bindArrows

try:
    import tkinter as tk  # python 3
    import tkinter.font as tk_Font
except:
    import Tkinter as tk  # python 2
    import tkFont as tk_Font


def choicebox(msg="Pick an item", title="", choices=[], preselect=0,
              callback=None,
              run=True):
    """
    Present the user with a list of choices.
    return the choice that he selects.

    :param str msg: the msg to be displayed
    :param str title: the window title
    :param list choices: a list or tuple of the choices to be displayed
    :param preselect: Which item, if any are preselected when dialog appears
    :return: List containing choice selected or None if cancelled
    """
    mb = ChoiceBox(msg, title, choices, preselect=preselect,
                   multiple_select=False,
                   callback=callback)
    if run:
        reply = mb.run()
        return reply
    else:
        return mb


def multchoicebox(msg="Pick an item", title="", choices=[],
                  preselect=0, callback=None,
                  run=True):
    """ Same as choicebox, but the user can select many items.

    """
    mb = ChoiceBox(msg, title, choices, preselect=preselect,
                   multiple_select=True,
                   callback=callback)
    if run:
        reply = mb.run()
        return reply
    else:
        return mb


# Utility function.  But, is it generic enough to be moved out of here?
def make_list_or_none(obj, cast_type=None):
    # -------------------------------------------------------------------
    # for an object passed in, put it in standardized form.
    # It may be None.  Just return None
    # If it is a scalar, attempt to cast it into cast_type.  Raise error
    # if not possible.  Convert scalar to a single-element list.
    # If it is a collections.Sequence (including a scalar converted to let),
    # then cast each element to cast_type.  Raise error if any cannot be converted.
    # -------------------------------------------------------------------
    ret_val = obj
    if ret_val is None:
        return None
    # Convert any non-sequence to single-element list
    if not isinstance(obj, collections.Sequence):
        if cast_type is not None:
            try:
                ret_val = cast_type(obj)
            except Exception as e:
                raise Exception("Value {} cannot be converted to type: {}".format(obj, cast_type))
        ret_val = [ret_val,]
    # Convert all elements to cast_type
    if cast_type is not None:
        try:
            ret_val = [cast_type(elem) for elem in ret_val]
        except:
            raise Exception("Not all values in {}\n can be converted to type: {}".format(ret_val, cast_type))
    return ret_val


class ChoiceBox(object):

    def __init__(self, msg, title, choices, preselect, multiple_select, callback):

        self.callback = callback

        self.choices = self.to_list_of_str(choices)

        # Convert preselect to always be a list or None.
        preselect_list = make_list_or_none(preselect, cast_type=int)
        if not multiple_select and len(preselect_list)>1:
            raise ValueError("Multiple selections not allowed, yet preselect has multiple values:{}".format(preselect_list))

        self.ui = GUItk(msg, title, self.choices, preselect_list, multiple_select,
                        self.callback_ui)

    def run(self):
        """ Start the ui """
        self.ui.run()
        self.ui = None
        return self.choices

    def stop(self):
        """ Stop the ui """
        self.ui.stop()

    def callback_ui(self, ui, command, choices):
        """ This method is executed when ok, cancel, or x is pressed in the ui.
        """
        if command == 'update':  # OK was pressed
            self.choices = choices
            if self.callback:
                # If a callback was set, call main process
                self.callback(self)
            else:
                self.stop()
        elif command == 'x':
            self.stop()
            self.choices = None
        elif command == 'cancel':
            self.stop()
            self.choices = None

    # methods to change properties --------------

    @property
    def msg(self):
        """Text in msg Area"""
        return self._msg

    @msg.setter
    def msg(self, msg):
        self.ui.set_msg(msg)

    @msg.deleter
    def msg(self):
        self._msg = ""
        self.ui.set_msg(self._msg)

    # Methods to validate what will be sent to ui ---------

    def to_list_of_str(self, choices):
        # -------------------------------------------------------------------
        # If choices is a tuple, we make it a list so we can sort it.
        # If choices is already a list, we make a new list, so that when
        # we sort the choices, we don't affect the list object that we
        # were given.
        # -------------------------------------------------------------------
        choices = list(choices)

        choices = [str(c) for c in choices]

        while len(choices) < 2:
            choices.append("Add more choices")

        return choices



class GUItk(object):

    """ This object contains the tk root object.
        It draws the window, waits for events and communicates them
        to MultiBox, together with the entered values.

        The position in wich it is drawn comes from a global variable.

        It also accepts commands from Multibox to change its message.
    """

    def __init__(self, msg, title, choices, preselect, multiple_select, callback):

        self.callback = callback

        self.choices = choices

        self.width_in_chars = global_state.prop_font_line_length
        # Initialize self.selected_choices
        # This is the value that will be returned if the user clicks the close
        # icon
        # self.selected_choices = None

        self.multiple_select = multiple_select

        self.boxRoot = tk.Tk()

        self.boxFont = tk_Font.Font(
            family=global_state.PROPORTIONAL_FONT_FAMILY,
            size=global_state.PROPORTIONAL_FONT_SIZE)

        self.boxFont = tk_Font.nametofont("TkTextFont")

        self.config_root(title)

        self.set_pos(global_state.window_position)  # GLOBAL POSITION

        self.create_msg_widget(msg)

        self.create_choicearea()

        self.create_ok_button()

        self.create_cancel_button()

        self. create_special_buttons()

        self.preselect_choice(preselect)

        self.choiceboxWidget.focus_force()



    # Run and stop methods ---------------------------------------

    def run(self):
        self.boxRoot.mainloop()  # run it!
        self.boxRoot.destroy()   # Close the window

    def stop(self):
        # Get the current position before quitting
        self.get_pos()

        self.boxRoot.quit()

    def x_pressed(self):
        self.callback(self, command='x', choices=self.get_choices())

    def cancel_pressed(self, event):
        self.callback(self, command='cancel', choices=self.get_choices())

    def ok_pressed(self, event):
        self.callback(self, command='update', choices=self.get_choices())

    # Methods to change content ---------------------------------------

    # Methods to change content ---------------------------------------

    def set_msg(self, msg):
        self.messageArea.config(state=tk.NORMAL)
        self.messageArea.delete(1.0, tk.END)
        self.messageArea.insert(tk.END, msg)
        self.messageArea.config(state=tk.DISABLED)
        # Adjust msg height
        self.messageArea.update()
        numlines = self.get_num_lines(self.messageArea)
        self.set_msg_height(numlines)
        self.messageArea.update()
        # put the focus on the entryWidget

    def set_msg_height(self, numlines):
        self.messageArea.configure(height=numlines)

    def get_num_lines(self, widget):
        end_position = widget.index(tk.END)  # '4.0'
        end_line = end_position.split('.')[0]  # 4
        return int(end_line) + 1  # 5

    def set_pos(self, pos=None):
        if not pos:
            pos = global_state.window_position
        self.boxRoot.geometry(pos)

    def get_pos(self):
        # The geometry() method sets a size for the window and positions it on
        # the screen. The first two parameters are width and height of
        # the window. The last two parameters are x and y screen coordinates.
        # geometry("250x150+300+300")
        geom = self.boxRoot.geometry()  # "628x672+300+200"
        global_state.window_position = '+' + geom.split('+', 1)[1]

    def preselect_choice(self, preselect):
        print(preselect)
        if preselect != None:
            for v in preselect:
                self.choiceboxWidget.select_set(v)
                self.choiceboxWidget.activate(v)

    def get_choices(self):
        choices_index = self.choiceboxWidget.curselection()
        if not choices_index:
            return None
        if self.multiple_select:
            selected_choices = [self.choiceboxWidget.get(index)
                                for index in choices_index]
        else:
            selected_choices = self.choiceboxWidget.get(choices_index)

        return selected_choices

    # Auxiliary methods -----------------------------------------------
    def calc_character_width(self):
        char_width = self.boxFont.measure('W')
        return char_width

    def config_root(self, title):

        screen_width = self.boxRoot.winfo_screenwidth()
        screen_height = self.boxRoot.winfo_screenheight()
        self.root_width = int((screen_width * 0.8))
        root_height = int((screen_height * 0.5))

        self.boxRoot.title(title)
        self.boxRoot.iconname('Dialog')
        self.boxRoot.expand = tk.NO
        # self.boxRoot.minsize(width=62 * self.calc_character_width())

        self.set_pos()

        self.boxRoot.protocol('WM_DELETE_WINDOW', self.x_pressed)
        self.boxRoot.bind('<Any-Key>', self.KeyboardListener)
        self.boxRoot.bind("<Escape>", self.cancel_pressed)

    def create_msg_widget(self, msg):

        if msg is None:
            msg = ""

        self.msgFrame = tk.Frame(
            self.boxRoot,
            padx=2 * self.calc_character_width(),

        )
        self.messageArea = tk.Text(
            self.msgFrame,
            width=self.width_in_chars,
            state=tk.DISABLED,
            background=self.boxRoot.config()["background"][-1],
            relief='flat',
            padx=(global_state.default_hpad_in_chars *
                  self.calc_character_width()),
            pady=(global_state.default_hpad_in_chars *
                  self.calc_character_width()),
            wrap=tk.WORD,
            font=('Arial',30)

        )
        self.set_msg(msg)

        self.msgFrame.pack(side=tk.TOP, expand=1, fill='both')

        self.messageArea.pack(side=tk.TOP, expand=1, fill='both')

    def create_choicearea(self):

        self.choiceboxFrame = tk.Frame(master=self.boxRoot)
        self.choiceboxFrame.pack(side=tk.TOP, fill=tk.BOTH, expand=tk.YES)

        lines_to_show = min(len(self.choices), 20)

        # --------  put the self.choiceboxWidget in the self.choiceboxFrame ---
        self.choiceboxWidget = tk.Listbox(self.choiceboxFrame,
                                          height=lines_to_show,
                                          borderwidth="1m", relief="flat",
                                          bg="white",
                                          font=('Arial',30)
                                          )

        if self.multiple_select:
            self.choiceboxWidget.configure(selectmode=tk.MULTIPLE)

        # self.choiceboxWidget.configure(font=(global_state.PROPORTIONAL_FONT_FAMILY,
        #                                      global_state.PROPORTIONAL_FONT_SIZE))

        # add a vertical scrollbar to the frame
        rightScrollbar = tk.Scrollbar(self.choiceboxFrame, orient=tk.VERTICAL,
                                      command=self.choiceboxWidget.yview)
        self.choiceboxWidget.configure(yscrollcommand=rightScrollbar.set)

        # add a horizontal scrollbar to the frame
        bottomScrollbar = tk.Scrollbar(self.choiceboxFrame,
                                       orient=tk.HORIZONTAL,
                                       command=self.choiceboxWidget.xview)
        self.choiceboxWidget.configure(xscrollcommand=bottomScrollbar.set)

        # pack the Listbox and the scrollbars.
        # Note that although we must define
        # the textArea first, we must pack it last,
        # so that the bottomScrollbar will
        # be located properly.

        bottomScrollbar.pack(side=tk.BOTTOM, fill=tk.X)
        rightScrollbar.pack(side=tk.RIGHT, fill=tk.Y)

        self.choiceboxWidget.pack(
            side=tk.LEFT, padx="1m", pady="1m", expand=tk.YES, fill=tk.BOTH)

        # Insert choices widgets
        for choice in self.choices:
            self.choiceboxWidget.insert(tk.END, choice)

        # Bind the keyboard events
        self.choiceboxWidget.bind("<Return>", self.ok_pressed)
        self.choiceboxWidget.bind("<Double-Button-1>",
                                  self.ok_pressed)

    def create_ok_button(self):

        self.buttonsFrame = tk.Frame(self.boxRoot)
        self.buttonsFrame.pack(side=tk.TOP, expand=tk.YES, pady=0)

        # put the buttons in the self.buttonsFrame
        okButton = tk.Button(self.buttonsFrame, takefocus=tk.YES,
                             text="OK", height=2, width=6)
        bindArrows(okButton)
        okButton.pack(expand=tk.NO, side=tk.RIGHT, padx='2m', pady='1m',
                      ipady="1m", ipadx="2m")

        # for the commandButton, bind activation events
        okButton.bind("<Return>", self.ok_pressed)
        okButton.bind("<Button-1>", self.ok_pressed)
        okButton.bind("<space>", self.ok_pressed)

    def create_cancel_button(self):
        cancelButton = tk.Button(self.buttonsFrame, takefocus=tk.YES,
                                 text="Cancel", height=1, width=6)
        bindArrows(cancelButton)
        cancelButton.pack(expand=tk.NO, side=tk.LEFT, padx='2m', pady='1m',
                          ipady="1m", ipadx="2m")
        cancelButton.bind("<Return>", self.cancel_pressed)
        cancelButton.bind("<Button-1>", self.cancel_pressed)
        # self.cancelButton.bind("<Escape>", self.cancel_pressed)
        # for the commandButton, bind activation events to the activation event
        # handler

    def create_special_buttons(self):
        # add special buttons for multiple select features
        if not self.multiple_select:
            return

        selectAllButton = tk.Button(
            self.buttonsFrame, text="Select All", height=1, width=6)
        selectAllButton.pack(expand=tk.NO, side=tk.LEFT, padx='2m',
                             pady='1m',
                             ipady="1m", ipadx="2m")

        clearAllButton = tk.Button(self.buttonsFrame, text="Clear All",
                                   height=1, width=6)
        clearAllButton.pack(expand=tk.NO, side=tk.LEFT,
                            padx='2m', pady='1m',
                            ipady="1m", ipadx="2m")

        selectAllButton.bind("<Button-1>", self.choiceboxSelectAll)
        bindArrows(selectAllButton)
        clearAllButton.bind("<Button-1>", self.choiceboxClearAll)
        bindArrows(clearAllButton)

    def KeyboardListener(self, event):
        key = event.keysym
        if len(key) <= 1:
            if key in string.printable:
                # Find the key in the liglobal_state.
                # before we clear the list, remember the selected member
                try:
                    start_n = int(self.choiceboxWidget.curselection()[0])
                except IndexError:
                    start_n = -1

                # clear the selection.
                self.choiceboxWidget.selection_clear(0, 'end')

                # start from previous selection +1
                for n in range(start_n + 1, len(self.choices)):
                    item = self.choices[n]
                    if item[0].lower() == key.lower():
                        self.choiceboxWidget.selection_set(first=n)
                        self.choiceboxWidget.see(n)
                        return
                else:
                    # has not found it so loop from top
                    for n, item in enumerate(self.choices):
                        if item[0].lower() == key.lower():
                            self.choiceboxWidget.selection_set(first=n)
                            self.choiceboxWidget.see(n)
                            return

                    # nothing matched -- we'll look for the next logical choice
                    for n, item in enumerate(self.choices):
                        if item[0].lower() > key.lower():
                            if n > 0:
                                self.choiceboxWidget.selection_set(
                                    first=(n - 1))
                            else:
                                self.choiceboxWidget.selection_set(first=0)
                            self.choiceboxWidget.see(n)
                            return

                    # still no match (nothing was greater than the key)
                    # we set the selection to the first item in the list
                    lastIndex = len(self.choices) - 1
                    self.choiceboxWidget.selection_set(first=lastIndex)
                    self.choiceboxWidget.see(lastIndex)
                    return

    def choiceboxClearAll(self, event):
        self.choiceboxWidget.selection_clear(0, len(self.choices) - 1)

    def choiceboxSelectAll(self, event):
        self.choiceboxWidget.selection_set(0, len(self.choices) - 1)

if __name__ == '__main__':
    users_choice = multchoicebox(choices=['choice1', 'choice2'])
    print("User's choice is: {}".format(users_choice))



"""

.. moduleauthor:: easygui developers and Stephen Raymond Ferg
.. default-domain:: py
.. highlight:: python

"""

import os
import sys

try:
    from . import utils as ut
    from .button_box import buttonbox
    from .text_box import textbox
    from .diropen_box import diropenbox
    from .fileopen_box import fileopenbox
    from .filesave_box import filesavebox
    from .multi_fillable_box import multenterbox
    from .multi_fillable_box import multpasswordbox

    from .derived_boxes import ynbox
    from .derived_boxes import ccbox
    from .derived_boxes import boolbox
    from .derived_boxes import indexbox
    from .derived_boxes import msgbox
    from .derived_boxes import integerbox
    from .derived_boxes import enterbox
    from .derived_boxes import exceptionbox
    from .derived_boxes import codebox
    from .derived_boxes import passwordbox

    from .choice_box import choicebox
    from .choice_box import multchoicebox

    from . import about
    from .about import eg_version
    from .about import abouteasygui
except (SystemError, ValueError, ImportError):
    print("Please run demo.py from outside the package")
    exit()
# --------------------------------------------------------------
#
# test/demo easygui
#
# -----------------------------------------------------------------------

package_dir = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))


class Demos(object):

    """ Collection of demos

        A choice is comprised of two pieces of data:
        - a description, which is a string. The descriptions will be shown
          in the choicebox, and one will be returned by it.
        - a function to execute when the description is selected
    """

    def __init__(self):
        self.demos = [
            ("msgbox", demo_msgbox),
            ("ynbox", demo_ynbox),
            ("ccbox", demo_ccbox),
            ("boolbox", demo_boolbox),
            ("buttonbox", demo_buttonbox),
            ("buttonbox that displays an image", demo_buttonbox_with_image),
            ("buttonbox - select an image", demo_buttonbox_with_choice),
            ("indexbox", demo_indexbox),
            ("choicebox", demo_choicebox),
            ("multchoicebox", demo_multichoicebox),
            ("textbox", demo_textbox),
            ("codebox", demo_codebox),
            ("enterbox", demo_enterbox),
            ("integerbox", demo_integerbox),
            ("passwordbox", demo_passwordbox),
            ("multenterbox", demo_multenterbox),
            ("multpasswordbox", demo_multpasswordbox),
            ("enterbox that displays an image", demo_enterbox_image),
            ("filesavebox", demo_filesavebox),
            ("fileopenbox", demo_fileopenbox),
            ("diropenbox", demo_diropenbox),
            ("exceptionbox", demo_exceptionbox),
            ("About EasyGui", demo_about),
            ("Help", demo_help),
        ]

    def list_descriptions(self):
        descriptions = [c[0] for c in self.demos]
        return descriptions

    def get_demo(self, index):
        demo = self.demos[index]
        # demo = next((c[1] for c in self.demo if c[0] == description))
        return demo[1]

    def get_description(self, index):
        demo = self.demos[index]
        return demo[0]

    def __len__(self):
        return len(self.demos)


def easygui_demo():
    """
    Run the EasyGui demo.
    """
    demos = Demos()
    replies = [''] * len(demos)
    # clear the console
    print('\n' * 100)

    msg = []
    msg.append("Pick the kind of box that you wish to demo.")
    msg.append(" * Python version {}".format(sys.version))
    msg.append(" * EasyGui version {}".format(eg_version))
    msg.append(" * Tk version {}".format(ut.TkVersion))
    intro_message = "\n".join(msg)
    title = "EasyGui " + eg_version
    # Table that relates keys in choicebox with functions to execute
    descriptions = demos.list_descriptions()
    preselected = 0
    while True:
        presented_choices = [
            d + r for d, r in zip(descriptions, replies)]
        reply = choicebox(msg=intro_message,
                          title=title,
                          choices=presented_choices,
                          preselect=preselected
                          )
        if not reply:
            break

        print(reply)

        index_chosen_demo = presented_choices.index(reply)

        # Execute the chosen demo!
        chosen_demo = demos.get_demo(index_chosen_demo)
        demo_reply = chosen_demo()

        # Save the reply
        if demo_reply:
            replies[index_chosen_demo] = ' - Last reply: {}'.format(demo_reply)
        else:
            replies[index_chosen_demo] = ''

        preselected += 1
        if preselected >= len(presented_choices):
            preselected = 0


def demo_msgbox():
    reply = msgbox("short msg", "This is a long title")
    print("Reply was: {!r}".format(reply))
    return reply


def demo_buttonbox():
    reply = buttonbox(choices=['one', 'two', 'two', 'three'],
                      default_choice='two')
    print("Reply was: {!r}".format(reply))

    title = "Demo of Buttonbox with many, many buttons!"
    msg = ("This buttonbox shows what happens when you "
           "specify too many buttons.")
    reply = buttonbox(msg=msg, title=title,
                      choices=['1', '2', '3', '4', '5', '6', '7'],
                      cancel_choice='7')
    print("Reply was: {!r}".format(reply))
    return reply


def demo_buttonbox_with_image():
    msg = "Do you like this picture?\nIt is "
    choices = ["Yes", "No", "No opinion"]

    for image in [
            os.path.join(package_dir, "python_and_check_logo.gif"),
            os.path.join(package_dir, "python_and_check_logo.jpg"),
            os.path.join(package_dir, "python_and_check_logo.png"),
            os.path.join(package_dir, "zzzzz.gif")]:
        reply = buttonbox(msg + image, image=image, choices=choices)
        print("Reply was: {!r}".format(reply))
    return reply

def demo_buttonbox_with_choice():
    msg = "Pick an image"
    choices = ['cancel']
    images = list()
    images.append(os.path.join(package_dir, "python_and_check_logo.gif"))
    images.append(os.path.join(package_dir, "python_and_check_logo.jpg"))
    images.append(os.path.join(package_dir, "python_and_check_logo.png"))
    images.append(os.path.join(package_dir, "zzzzz.gif"))
    reply = buttonbox(msg, images=images, choices=['cancel'])
    print("Reply was: {!r}".format(reply))
    return reply


def demo_ccbox():
    msg = "Insert your favorite message here"
    title = "Demo of ccbox"
    reply = ccbox(msg, title)
    print("Reply was: {!r}".format(reply))
    return reply


def demo_multichoicebox():
    listChoices = ["aaa", "bbb", "ccc", "ggg", "hhh", "iii",
                   "jjj", "kkk", "LLL", "mmm", "nnn", "ooo",
                   "ppp", "qqq", "rrr", "sss", "ttt", "uuu",
                   "vvv"]
    preselect = None #[0, 2, 4]
    msg = "Pick as many choices as you wish."
    reply = multchoicebox(msg, "Demo of multchoicebox", listChoices, preselect)
    print("Reply was: {!r}".format(reply))
    return reply


def demo_ynbox():
    title = "Demo of ynbox"
    msg = "Were you expecting the Spanish Inquisition?"
    reply = ynbox(msg, title)
    print("Reply was: {!r}".format(reply))
    if reply:
        msgbox("NOBODY expects the Spanish Inquisition!", "Wrong!")
    return reply


def demo_choicebox():
    title = "Demo of choicebox"
    longchoice = (
        "This is an example of a very long option "
        "which you may or may not wish to choose."
        * 2)
    listChoices = ["nnn", "ddd", "eee", "fff", "aaa",
                   longchoice, "aaa", "bbb", "ccc", "ggg", "hhh",
                   "iii", "jjj", "kkk", "LLL", "mmm", "nnn",
                   "ooo", "ppp", "qqq",
                   "rrr", "sss", "ttt", "uuu", "vvv"]

    msg = ("Pick something. " +
           ("A wrapable sentence of text ?! " * 30) +
           "\nA separate line of text." * 6)
    reply = choicebox(msg=msg, choices=listChoices)
    print("Reply was: {!r}".format(reply))

    msg = "Pick something. "
    reply = choicebox(msg=msg, title=title, choices=listChoices)
    print("Reply was: {!r}".format(reply))

    msg = "Pick something. "
    reply = choicebox(
        msg="The list of choices is empty!", choices=list())
    print("Reply was: {!r}".format(reply))
    return reply


def demo_integerbox():
    reply = integerbox(
        "Enter a number between 3 and 333",
        "Demo: integerbox WITH a default value", 222, 3, 333)
    print("Reply was: {!r}".format(reply))

    reply = integerbox(
        "Enter a number between 0 and 99",
        "Demo: integerbox WITHOUT a default value"
    )
    print("Reply was: {!r}".format(reply))
    return reply


def demo_about():
    reply = abouteasygui()
    print("Reply was: {!r}".format(reply))
    return reply


def demo_enterbox():
    image = os.path.join(package_dir, "python_and_check_logo.gif")
    message = ("Enter the name of your best friend."
               "\n(Result will be stripped.)")
    reply = enterbox(message, "Love!", "     Suzy Smith     ")
    print("Reply was: {!r}".format(reply))

    message = ("Enter the name of your best friend."
               "\n(Result will NOT be stripped.)")
    reply = enterbox(
        message, "Love!", "     Suzy Smith     ", strip=False)
    print("Reply was: {!r}".format(reply))

    reply = enterbox("Enter the name of your worst enemy:", "Hate!")
    print("Reply was: {!r}".format(reply))
    return reply


def demo_multpasswordbox():
    msg = "Enter logon information"
    title = "Demo of multpasswordbox"
    fieldNames = ["Server ID", "User ID", "Password"]
    fieldValues = list()  # we start with blanks for the values
    fieldValues = multpasswordbox(msg, title, fieldNames)

    # make sure that none of the fields was left blank
    while True:
        if fieldValues is None:
            break
        errs = list()
        for n, v in zip(fieldNames, fieldValues):
            if v.strip() == "":
                errs.append('"{}" is a required field.\n\n'.format(n))
        if not len(errs):
            break  # no problems found
        fieldValues = multpasswordbox(
            "".join(errs), title, fieldNames, fieldValues)

    print("Reply was: {!s}".format(fieldValues))
    return fieldValues


def demo_textbox():
    text_snippet = ((
        "It was the best of times, and it was the worst of times.  The rich "
        "ate cake, and the poor had cake recommended to them, but wished "
        "only for enough cash to buy bread.  The time was ripe for "
        "revolution! "
        * 5) + "\n\n") * 10
    title = "Demo of textbox"
    msg = "Here is some sample text. " * 16
    reply = textbox(msg, title, text_snippet)
    print("Reply was: {!s}".format(reply))
    return reply


def demo_codebox():
    # TODO RL: Turn this sample code into the code in this module, just for fun
    code_snippet = ("dafsdfa dasflkj pp[oadsij asdfp;ij asdfpjkop asdfpok asdfpok asdfpok" * 3) + "\n" + """# here is some dummy Python code
for someItem in myListOfStuff:
    do something(someItem)
    do something()
    do something()
    if somethingElse(someItem):
        doSomethingEvenMoreInteresting()

""" * 16
    msg = "Here is some sample code. " * 16
    reply = codebox(msg, "Code Sample", code_snippet)
    print("Reply was: {!r}".format(reply))
    return reply


def demo_boolbox():
    reply = boolbox()
    print("Reply was: {!r}".format(reply))
    return reply


def demo_enterbox_image():
    image = os.path.join(package_dir, "python_and_check_logo.gif")
    message = "What kind of snake is this?"
    reply = enterbox(message, "Quiz", image=image)
    print("Reply was: {!r}".format(reply))
    return reply


def demo_passwordbox():
    reply = passwordbox("Demo of password box WITHOUT default"
                        + "\n\nEnter your secret password",
                        "Member Logon")
    print("Reply was: {!s}".format(reply))

    reply = passwordbox("Demo of password box WITH default"
                        + "\n\nEnter your secret password",
                        "Member Logon", "alfie")
    print("Reply was: {!s}".format(reply))
    return reply


def demo_help():
    codebox("EasyGui Help", text=about.EASYGUI_ABOUT_INFORMATION)
    return None


def demo_filesavebox():
    filename = "myNewFile.txt"
    title = "File SaveAs"
    msg = "Save file as:"

    f = filesavebox(msg, title, default=filename)
    print("You chose to save file: {}".format(f))
    return f


def demo_diropenbox():
    title = "Demo of diropenbox"
    msg = "Pick the directory that you wish to open."
    d = diropenbox(msg, title)
    print("You chose directory...: {}".format(d))

    d = diropenbox(msg, title, default="./")
    print("You chose directory...: {}".format(d))

    d = diropenbox(msg, title, default="c:/")
    print("You chose directory...: {}".format(d))
    return d


def demo_exceptionbox():
    try:
        thisWillCauseADivideByZeroException = 1 / 0
    except:
        exceptionbox()
    return None


def demo_indexbox():
    title = "Indexbox"
    msg = "Demo of " + "indexbox"
    choices = ["Choice1", "Choice2", "Choice3", "Choice4"]
    reply = indexbox(msg, title, choices)
    print("Reply was: {!r}".format(reply))
    return reply


def demo_multenterbox():
    msg = "Enter your personal information"
    title = "Credit Card Application"
    fieldNames = ["Name", "Street Address", "City", "State", "ZipCode"]
    fieldValues = list()  # we start with blanks for the values
    fieldValues = multenterbox(msg, title, fieldNames)

    # make sure that none of the fields was left blank
    while True:
        if fieldValues is None:
            break
        errs = list()
        for n, v in zip(fieldNames, fieldValues):
            if v.strip() == "":
                errs.append('"{}" is a required field.'.format(n))
        if not len(errs):
            break  # no problems found
        fieldValues = multenterbox(
            "\n".join(errs), title, fieldNames, fieldValues)

    print("Reply was: {}".format(fieldValues))
    return fieldValues


def demo_fileopenbox():
    msg = "Python files"
    title = "Open files"
    default = "*.py"
    f = fileopenbox(msg, title, default=default)
    print("You chose to open file: {}".format(f))

    default = "./*.gif"
    msg = "Some other file types (Multi-select)"
    filetypes = ["*.jpg", ["*.zip", "*.tgs", "*.gz",
                           "Archive files"], ["*.htm", "*.html", "HTML files"]]
    f = fileopenbox(
        msg, title, default=default, filetypes=filetypes, multiple=True)
    print("You chose to open file: %s" % f)
    return f



"""

.. moduleauthor:: easygui developers and Stephen Raymond Ferg
.. default-domain:: py
.. highlight:: python

Version |release|
"""


import os

try:
    from . import utils as ut
    from . import fileboxsetup as fbs
except (SystemError, ValueError, ImportError):
    import utils as ut
    import fileboxsetup as fbs

try:
    import tkinter as tk  # python 3
    import tkinter.font as tk_Font
except:
    import Tkinter as tk  # python 2
    import tkFont as tk_Font



# -------------------------------------------------------------------
# filesavebox
# -------------------------------------------------------------------


def filesavebox(msg=None, title=None, default="", filetypes=None):
    """
    A file to get the name of a file to save.
    Returns the name of a file, or None if user chose to cancel.

    The "default" argument should contain a filename (i.e. the
    current name of the file to be saved).  It may also be empty,
    or contain a filemask that includes wildcards.

    The "filetypes" argument works like the "filetypes" argument to
    fileopenbox.

    :param str msg: the msg to be displayed.
    :param str title: the window title
    :param str default: default filename to return
    :param object filetypes: filemasks that a user can choose, e.g. " \*.txt"
    :return: the name of a file, or None if user chose to cancel
    """

    localRoot = tk.Tk()
    localRoot.withdraw()

    initialbase, initialfile, initialdir, filetypes = fbs.fileboxSetup(
        default, filetypes)

    f = ut.tk_FileDialog.asksaveasfilename(
        parent=localRoot,
        title=ut.getFileDialogTitle(
            msg, title),
        initialfile=initialfile, initialdir=initialdir,
        filetypes=filetypes
    )
    localRoot.destroy()
    if not f:
        return None
    return os.path.normpath(f)


if __name__ == '__main__':
    print("Hello from file save box")
    ret_val = filesavebox("Please select a file to save to", "My File Save dialog")
    print("Return value is:{}".format(ret_val))
